
#define CLK "17"
#define LTH "27"
#define DIN "22"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>

int phrasenum;
char *phrase [] = {"xdotool type 'man omxplayer'", "xdotool type 'man sox'", 
"xdotool type 'play %f gain -2'", "xdotool type 'omxplayer -b --adev=alsa %f'",
"avplay %f", NULL}; 

void out (char *fname, char *text) {
  FILE *f = fopen(fname, "w");
  if (!f) {perror(fname); return;}
  fprintf(f, "%s", text);
  fclose(f);
}

FILE *clkp, *lthp, *dinp;

void outval (int sel, char *text) {
  FILE **f = (sel == 27) ? &lthp : &clkp;
  if (!(*f)) *f = fopen((sel == 27) ? "/sys/class/gpio/gpio" LTH "/value" : "/sys/class/gpio/gpio" CLK "/value", "w");
  if (!(*f)) {perror("outval"); return;}
  fprintf(*f, "%s", text);
}

int read_pin (char *fname) {
  if (!dinp) dinp = fopen(fname, "r");
  if (!dinp) {perror(fname); return 1;}
  char buf [2];
  fread(buf, 1, 1, dinp);
  fclose(dinp); dinp = NULL;
  if (buf[0] == '1') return 1;
  return 0;
}

int key [200], keydown [200], i, mode = 3;
char b [400];

struct timespec ti;

void wait_a_bit () {
  ti.tv_sec = 0;
  ti.tv_nsec = 900000;
  nanosleep(&ti, NULL);
}

void scan (void) {
  out("/sys/class/gpio/gpio" CLK "/value", "0");
  wait_a_bit();
  out("/sys/class/gpio/gpio" LTH "/value", "0");
  wait_a_bit();
  if (mode == 3) sleep(1);
  out("/sys/class/gpio/gpio" LTH "/value", "1");
  wait_a_bit();
  int i;
  for (i = 0; i < 16; i++) {
    key[i] = !read_pin("/sys/class/gpio/gpio" DIN "/value"); /* invert logic */
    wait_a_bit();
    out("/sys/class/gpio/gpio" CLK "/value", "1");
    wait_a_bit();
    out("/sys/class/gpio/gpio" CLK "/value", "0");
  }
}

static void handler(int sig, siginfo_t *si, void *unused) {
  fprintf(stderr, "signal...\n\n");
  out("/sys/class/gpio/unexport", CLK); /* clock   */
  out("/sys/class/gpio/unexport", LTH); /* latch   */
  out("/sys/class/gpio/unexport", DIN); /* data in */
  exit(0);
}

enum {KEY, REPEAT, KEYDOWN_KEYUP};

int check_key (int num, int type) {
  int ret = 0;
  if (type == KEY && key[num] && !keydown[num]) ret = 1;
  if (type == REPEAT && key[num]) return 1;
  if (type == KEYDOWN_KEYUP && key[num] != keydown[num]) ret = 1;
  keydown[num] = key[num];
  return ret;
}

int main (void) {
  struct sigaction sa;
  sa.sa_flags = SA_SIGINFO;
  sigemptyset(&sa.sa_mask);
  sa.sa_sigaction = handler;
  sigaction(SIGINT, &sa, NULL);
  sigaction(SIGTERM, &sa, NULL);
  out("/sys/class/gpio/export", CLK); /* clock   */
  out("/sys/class/gpio/export", LTH); /* latch   */
  out("/sys/class/gpio/export", DIN); /* data in */
  out("/sys/class/gpio/gpio" CLK "/direction", "out");
  out("/sys/class/gpio/gpio" LTH "/direction", "out");
  out("/sys/class/gpio/gpio" DIN "/direction", "in");
  out("/sys/class/gpio/gpio" LTH "/value", "1");
  wait_a_bit();
  while(1) {
    scan();
    /* for(i = 0; i < 16; i++) fprintf(stderr, "%i", key[i]);
    fprintf(stderr, "\n"); */
    
    if (key[9] && !keydown[9]) {
      if (key[4]) { sleep(2); continue; } /* connection broken */
      mode++;
      mode &= 3; /* modulo 4 */
      if (key[7]) system("shutdown now");
    }
    keydown[9] = key[9];
    
    if (key[4] && !keydown[4]) {
      /* slow mouse modifier or space */
      if (key[7] && key[4]) system("xdotool key Escape");
    }
    keydown[4] = key[4];
    
    if (key[6] && !keydown[6]) {
      if (key[7] && key[6]) system("xdotool key CONTROL+c");
    }
    keydown[4] = key[4];
    
    if (key[7] && key[2]) system("reboot");
    
    if (check_key(10, KEYDOWN_KEYUP)) { /* ALT */
      snprintf(b, sizeof(b), "xdotool key%s ALT", (key[10]) ? "down" : "up");
      system(b); /* run command */
    }
    
    if (mode == 0) {
      int dx = 0, dy = 0, m = (key[4]) ? 4 : 70;
      if (key[5]) dx = -m;
      if (key[1]) dx = +m;
      if (key[3]) dy = -m;
      if (key[0]) dy = +m;
      fprintf(stderr, "moving %i %i\n", dx, dy);
      snprintf(b, sizeof(b), "xdotool mousemove_relative -- %i %i", dx, dy);
      if (dx || dy) system(b); /* run command */
      
      if (check_key(2, KEYDOWN_KEYUP)) { /* mouse left */
        snprintf(b, sizeof(b), "xdotool mouse%s 1", (key[2]) ? "down" : "up");
        system(b); /* run command */
      }
      if (check_key(15, KEYDOWN_KEYUP)) { /* mouse right */
        snprintf(b, sizeof(b), "xdotool mouse%s 3", (key[15]) ? "down" : "up");
        system(b); /* run command */
      }
      
      
    } else if (mode & 1) { /* modes 1 and 3 */
      if (check_key(2,  KEY)) system("xdotool key Return");
      if (check_key(5,  KEY)) system("xdotool key Left");
      if (check_key(1,  KEY)) system("xdotool key Right");
      if (check_key(3,  KEY)) system("xdotool key Up");
      if (check_key(0,  KEY)) system("xdotool key Down");
      if (check_key(15, KEY)) system("xdotool key BackSpace");
      if (check_key(2,  KEY)) system("xdotool key Return");
      /* if (check_key(14, KEY)) {
        if (!phrase[phrasenum] || phrase[phrasenum][0] == '\0') phrasenum = 0;
        system(phrase[phrasenum]);
        phrasenum++;
      } */
      if (check_key(8,  KEY)) system("xdotool key space");
      if (check_key(12, KEY)) system("xdotool key k"); /* audio */
      if (check_key(13, KEY)) system("xdotool key m"); /* subtitles */
      if (check_key(14, KEY)) system("xdotool key n"); /* subtitles */
      if (check_key(11, KEY)) system("xdotool key Tab");
    }
    
    wait_a_bit();
  }
  return 0;
}
