/* This program is public domain. Use it as you wish */

/* gpio userspace keyboard */

/* use this command to compile:
   gcc rpi_control.c -lwiringPi -o rpi_control */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <err.h>
#include <wiringPi.h>

#define D  25
#define DI 24
#define C  23
#define SS 21

unsigned char c [8];
int ii;
int graceperiod = 10;
int gracereset = 5;
int need_reset = 1;
int pre [8], keys [8], hold;
char comm [128];
int led = 0;
int mode = 0;
int blinkk = 0;
int led_err = 0;
int msh = 0;
int gracekeys = 0;
int note = 5;

int checkparity (void) {
  if (c[7] & 0b10000000) return 1; /* bad byte */
  int i = !!(c[7] & 0b01000000);
  i ^= !!(c[7] & 0b00100000);
  i ^= !!(c[7] & 0b00010000);
  i ^= !!(c[7] & 0b00001000);
  i ^= !!(c[7] & 0b00000100);
  i ^= !!(c[7] & 0b00000010);
  i ^= !!(c[7] & 0b00000001);
  return !i; /* parity bit was inverted, yes */
}


void process_modifier (void) {
  if (keys[5] && !pre[5]) {
    mode++;
    if (mode > 3) mode = 0;
    warnx("mode = %i", mode);
  }
  
  if (0 == mode) {
    if (keys[6] && !pre[6]) system(
      "xdotool click 4");
    if (keys[3] && !pre[3]) system(
      "xdotool click 5");
    if (keys[1] && !pre[1]) system(
      "xdotool key Tab");
    if (keys[2] && !pre[2]) system(
      "xdotool key Escape");
    if (keys[4] && !pre[4]) system(
      "xdotool keydown ALT");
    if (!keys[4] && pre[4]) system(
      "xdotool keyup ALT");
    if (keys[7] && !pre[7]) system(
      "xdotool keydown CONTROL");
    if (!keys[7] && pre[7]) system(
      "xdotool keyup CONTROL");
      
  } else if (1 == mode) {
    if (keys[4] && !pre[4]) system(
      "sudo shutdown now");
    //if (keys[3] && !pre[3]) system(
    //  "xdotool key CONTROL+c");
    /* if (keys[6] && !pre[6]) system(
      "xdotool key ALT+F11"); */

    if (keys[6] && !pre[6])
      system("xdotool key CONTROL+SHIFT+equal");
    if (keys[3] && !pre[3])
      system("xdotool key CONTROL+minus");
      
  } else if (2 == mode) {
    if (keys[1] && !pre[1]) msh = 1;
    if (keys[6] && !pre[6])
      system("xdotool key Up");
    if (keys[3] && !pre[3])
      system("xdotool key Down");
    if (keys[7] && !pre[7])
      system("xdotool key CONTROL+Up");
    if (keys[4] && !pre[4])
      system("xdotool key CONTROL+Down");
    
    if (keys[2] && !pre[2]) {
      note++;
      if (note > 7) note = 1;
      char *cmdd = "";
      if (7 == note) cmdd = "xdotool key 7";
      if (6 == note) cmdd = "xdotool key 6";
      if (5 == note) cmdd = "xdotool key 5";
      if (4 == note) cmdd = "xdotool key 4";
      if (3 == note) cmdd = "xdotool key 3";
      if (2 == note) cmdd = "xdotool key 2";
      if (1 == note) cmdd = "xdotool key 1";
      system(cmdd);
    }
  }
}


void process_0 (void) {
  int mm = 0, x = 0, y = 0;
  
  if (keys[2] && !pre[2]) system(
  "xdotool keydown ALT");

  if (!keys[2] && pre[2]) system(
  "xdotool keyup   ALT");
  
  if (keys[4] && !pre[4]) system(
  "xdotool mousedown 1");
  
  if (pre[4] && !keys[4]) system(
  "xdotool mouseup 1");
  
  if (keys[6] && !pre[6]) system(
  "xdotool mousedown 3");
  
  if (pre[6] && !keys[6]) system(
  "xdotool mouseup 3");
  
  if (keys[1]) {
    hold++;
    y = (hold > 20) ? 30 : 10;
    if (!pre[1]) {
      mm = 1;
    } else if (hold > 10) mm = 1;
  }
  if (keys[3]) {
    hold++;
    x = (hold > 20) ? 30 : 10;
    if (!pre[3]) {
      mm = 1;
    } else if (hold > 10) mm = 1;
  }
  if (keys[5]) {
    hold++;
    x = (hold > 20) ? -30 : -10;
    if (!pre[5]) {
      mm = 1;
    } else if (hold > 10) mm = 1;
  }
  if (keys[7]) {
    hold++;
    y = (hold > 20) ? -30 : -10;
    if (!pre[7]) {
      mm = 1;
    } else if (hold > 10) mm = 1;
  }
  
  if (!keys[1] && !keys[3] &&
      !keys[5] && !keys[7]) hold = 0;
  
  if (mm) {
    if (!blinkk) blinkk = 2;
    snprintf(comm, sizeof(comm),
    "xdotool mousemove_relative -- %i %i",
    x, y);
    system(comm);
  }
}


void process_1 (void) {
  if (keys[1] && !pre[1]) system(
  "xdotool key Down");
  if (keys[2] && !pre[2]) system(
  "xdotool key space");
  if (keys[3] && !pre[3]) system(
  "xdotool key Right");
  if (keys[4] && !pre[4]) system(
  "xdotool key Return");
  if (keys[5] && !pre[5]) system(
  "xdotool key Left");
  if (keys[6] && !pre[6]) system(
  "xdotool key f");
  if (keys[7] && !pre[7]) system(
  "xdotool key Up");
  
  
}

void process_2 (void) {
  if (gracekeys) return;

  if (keys[7] && keys[6]) { /* multi */
    system("xdotool key n"); /* musescore */
    gracekeys = 1;
  }
  if (keys[4] && keys[3]) { /* multi */
    system("xdotool key 0"); /* musescore */
    gracekeys = 1;
  }
  if (keys[5] && keys[4]) { /* multi */
    system("xdotool key CONTROL+z"); /* musescore */
    gracekeys = 1;
  }

  if (!keys[7] && pre[7]) {
    system((msh) ? "xdotool key D" : "xdotool key d");
    msh = 0;
  }
  if (!keys[6] && pre[6]) {
    system((msh) ? "xdotool key E" : "xdotool key e");
    msh = 0;
  }
  if (!keys[5] && pre[5]) {
    system((msh) ? "xdotool key F" : "xdotool key f");
    msh = 0;
  }
  if (!keys[4] && pre[4]) {
    system((msh) ? "xdotool key G" : "xdotool key g");
    msh = 0;
  }
  if (!keys[3] && pre[3]) {
    system((msh) ? "xdotool key A" : "xdotool key a");
    msh = 0;
  }
  if (!keys[2] && pre[2]) {
    system((msh) ? "xdotool key B" : "xdotool key b");
    msh = 0;
  }
  if (!keys[1] && pre[1]) {
    system((msh) ? "xdotool key C" : "xdotool key c");
    msh = 0;
  }
  
}


int main (void) {
  if (system("pidof rpi_control")) errx(
    1, "another driver running");
  wiringPiSetup();
  pinMode(SS, OUTPUT);  /* SPI Select */
  pinMode(D , OUTPUT);  /* MOSI  */
  pinMode(DI, INPUT );  /* MISO  */
  pinMode(C , OUTPUT);  /* Clock */
  digitalWrite(SS, LOW);
  delay(5);

  while (1) {
    led = (mode + 1) | led_err;
    if (blinkk) {
      blinkk--;
      led = 7;
    }
    
    delay(4);
    c[7] = 0;
    for (ii = 0; ii < 8; ii++) {
      digitalWrite(D, led &(1 << 7 - ii));
      digitalWrite(C, LOW );
      delay(1);
      digitalWrite(C, HIGH);
      delay(1);
      if (digitalRead(DI))
        c[7] |= (1 << 7 - ii);
    }

    /* fprintf(stderr, " %i%i%i%i%i%i%i%i",
      !!(c[7] & 0b10000000),
      !!(c[7] & 0b01000000),
      !!(c[7] & 0b00100000),
      !!(c[7] & 0b00010000),
      !!(c[7] & 0b00001000),
      !!(c[7] & 0b00000100),
      !!(c[7] & 0b00000010),
      !!(c[7] & 0b00000001)
      );
    if (c[7] & 0b00010000)
      fprintf(stderr, "\n"); */
    
    int par = c[7] & 0b01000000;
    int nibble = c[7] & 0b00010000;
    for (ii = 0; ii < 4; ii++) {
      keys[(nibble) ? ii + 4 : ii]
        = !!(c[7] & (1 << ii));
    }
    
    led_err = 0;
    if (checkparity()) {
      graceperiod = 30;
      if (!gracereset) {
        gracereset = 5;
        need_reset = 1;
        led_err = 0b00100000; /* red light */
      }
    }
    
    if (need_reset) {
      warnx("spi chip select pulse");
      digitalWrite(SS, HIGH);
      delay(5);
      digitalWrite(SS, LOW);
      delay(5);
      need_reset = 0;
    }
    
    if (!pre[0] && !pre[1] &&
      !pre[2] && !pre[3] &&
      !pre[4] && !pre[5] &&
      !pre[6] && !pre[7]) gracekeys = 0;
    
    if (!graceperiod) {
      if (keys[0]) process_modifier();
      else if (0 == mode) process_0();
      else if (1 == mode) process_1();
      else if (2 == mode) process_2();
    }
    memcpy(pre, keys, sizeof(keys));
    
    if (graceperiod > 0) graceperiod--;
    if (gracereset > 0) gracereset--;
  }
  
  return 0;
}

