module top(						//top module
	CLK,
	BUT1,
	BUT2,
	LED1,
	LED2,
	EXT_PIO2_1,
	EXT_PIO2_2,
	EXT_PIO2_3,
	EXT_PIO2_4
);

input			CLK;				//input 100Mhz clock
input			BUT1;				//input signal from button 1
input			BUT2;				//input signal from button 2
output			LED1;				//output signal to LED1
output			LED2;				//output signal to LED2
input			EXT_PIO2_1;  // clock
input			EXT_PIO2_2;  // data in
output			EXT_PIO2_3;  // data out
input			EXT_PIO2_4;  // strobe (inverted) (external upll-up needed)

reg shift_din_latched;					//shift register 'data in'
reg shift_clk_latched;
reg [1:0] shift_state_machine;				//shift register state machine
reg [7:0] shift_debounce_timer;				//shift register debounce timer
reg [15:0] shift_reg;					//shift register itself
reg [15:0] shift_latched;

assign EXT_PIO2_3 = shift_reg[15]; // output data wire
assign LED1 = shift_latched[2];
assign LED2 = shift_latched[3];
assign strobe_or_clock = shift_clk_latched | (~shift_strobe_latched);

always @ (posedge CLK) begin
  shift_din_latched <= EXT_PIO2_2;	// input data  wire
  shift_clk_latched <= EXT_PIO2_1;	// input clock wire
  shift_strobe_latched <= EXT_PIO2_4;

  case (shift_state_machine)
    2'b00 :
      if (strobe_or_clock) begin
        shift_state_machine <= 2'b01;
        if (shift_clk_latched) shift_reg <= {shift_reg[14:0], shift_din_latched};
        else if (!shift_strobe_latched) begin
          shift_latched <= shift_reg;
          shift_reg[15:8] <= {4'b0000, (~BUT1), (~BUT2), 2'b00}; // output buttons
        end
      end
    2'b01 : if (!strobe_or_clock) shift_state_machine <= 2'b10;
    
    2'b10 : shift_state_machine <= 2'b00;
    
    default : shift_state_machine <= 2'b00;
  endcase
end

endmodule

